@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013 Brazil
+  Copyright(C) 2013-2014 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -20,14 +20,19 @@
 #include "ctx_impl.h"
 
 #include "mrb.h"
+#include "mrb/mrb_id.h"
+#include "mrb/mrb_operator.h"
 #include "mrb/mrb_ctx.h"
+#include "mrb/mrb_logger.h"
+#include "mrb/mrb_void.h"
+#include "mrb/mrb_bulk.h"
 #include "mrb/mrb_obj.h"
 #include "mrb/mrb_column.h"
 #include "mrb/mrb_fixed_size_column.h"
 #include "mrb/mrb_variable_size_column.h"
+#include "mrb/mrb_index_column.h"
 #include "mrb/mrb_expr.h"
 #include "mrb/mrb_accessor.h"
-#include "mrb/mrb_index_info.h"
 #include "mrb/mrb_procedure.h"
 
 #ifdef GRN_WITH_MRUBY
@@ -39,14 +44,21 @@ grn_ctx_impl_mrb_init_bindings(grn_ctx *ctx)
   mrb->ud = ctx;
   ctx->impl->mrb.module = mrb_define_module(mrb, "Groonga");
 
+  grn_mrb_load(ctx, "backtrace_entry.rb");
+
+  grn_mrb_id_init(ctx);
+  grn_mrb_operator_init(ctx);
   grn_mrb_ctx_init(ctx);
+  grn_mrb_logger_init(ctx);
+  grn_mrb_void_init(ctx);
+  grn_mrb_bulk_init(ctx);
   grn_mrb_obj_init(ctx);
   grn_mrb_column_init(ctx);
   grn_mrb_fixed_size_column_init(ctx);
   grn_mrb_variable_size_column_init(ctx);
+  grn_mrb_index_column_init(ctx);
   grn_mrb_expr_init(ctx);
   grn_mrb_accessor_init(ctx);
-  grn_mrb_index_info_init(ctx);
   grn_mrb_procedure_init(ctx);
 }
 
