@@ -128,6 +128,8 @@ struct _grn_type {
   grn_db_obj obj;
 };
 
+#define GRN_TYPE_SIZE(type) ((type)->range)
+
 #define GRN_TABLE_SORT_GEO            (0x02<<0)
 
 #define GRN_OBJ_TMP_OBJECT 0x80000000
@@ -156,10 +158,6 @@ struct _grn_type {
   (GRN_OBJ_VECTOR_COLUMNP(obj) &&\
    (DB_OBJ(obj)->header.flags & GRN_OBJ_WITH_WEIGHT))
 
-typedef grn_rc grn_selector_func(grn_ctx *ctx, grn_obj *table, grn_obj *index,
-                                 int nargs, grn_obj **args,
-                                 grn_obj *res, grn_operator op);
-
 typedef struct _grn_proc_ctx grn_proc_ctx;
 
 struct _grn_proc_ctx {
@@ -208,9 +206,6 @@ GRN_API grn_obj *grn_proc_get_or_add_var(grn_ctx *ctx, grn_user_data *user_data,
 GRN_API grn_obj *grn_proc_alloc(grn_ctx *ctx, grn_user_data *user_data,
                                 grn_id domain, grn_obj_flags flags);
 
-grn_rc grn_proc_set_selector(grn_ctx *ctx, grn_obj *proc,
-                             grn_selector_func selector);
-
 GRN_API grn_rc grn_proc_call(grn_ctx *ctx, grn_obj *proc,
                              int nargs, grn_obj *caller);
 
@@ -423,6 +418,15 @@ void grn_obj_spec_save(grn_ctx *ctx, grn_db_obj *obj);
 
 grn_rc grn_obj_reinit_for(grn_ctx *ctx, grn_obj *obj, grn_obj *domain_obj);
 
+#define GRN_INT32_POP(obj,value) do {\
+  if (GRN_BULK_VSIZE(obj) >= sizeof(int32_t)) {\
+    GRN_BULK_INCR_LEN((obj), -(sizeof(int32_t)));\
+    value = *(int32_t *)(GRN_BULK_CURR(obj));\
+  } else {\
+    value = 0;\
+  }\
+} while (0)
+
 #define GRN_UINT32_POP(obj,value) do {\
   if (GRN_BULK_VSIZE(obj) >= sizeof(uint32_t)) {\
     GRN_BULK_INCR_LEN((obj), -(sizeof(uint32_t)));\
