@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2013 Brazil
+  Copyright(C) 2009-2014 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -48,6 +48,10 @@
 # include <sys/sysctl.h>
 #endif /* HAVE_SYS_SYSCTL_H */
 
+#ifdef HAVE__STRNICMP
+# define strncasecmp(s1,s2,n) _strnicmp(s1,s2,n)
+#endif /* HAVE__STRNICMP */
+
 #ifndef USE_MSG_NOSIGNAL
 # ifdef MSG_NOSIGNAL
 #  undef MSG_NOSIGNAL
@@ -65,7 +69,8 @@
 # define STDERR_FILENO 2
 #endif /* STDERR_FILENO */
 
-#define DEFAULT_PORT 10041
+#define DEFAULT_HTTP_PORT 10041
+#define DEFAULT_GQTP_PORT 10043
 #define DEFAULT_DEST "localhost"
 #define DEFAULT_MAX_NFTHREADS 8
 #define MAX_CON 0x10000
@@ -74,11 +79,10 @@
 
 static char bind_address[HOST_NAME_MAX + 1];
 static char hostname[HOST_NAME_MAX + 1];
-static int port = DEFAULT_PORT;
+static int port = DEFAULT_GQTP_PORT;
 static int batchmode;
 static int number_of_lines = 0;
 static int newdb;
-static int useql;
 static grn_bool is_daemon_mode = GRN_FALSE;
 static int (*do_client)(int argc, char **argv);
 static int (*do_server)(char *path);
@@ -260,12 +264,6 @@ prompt(grn_ctx *ctx, grn_obj *buf)
   return rc;
 }
 
-typedef enum {
-  grn_http_request_type_none = 0,
-  grn_http_request_type_get,
-  grn_http_request_type_post
-} grn_http_request_type;
-
 static void
 output_envelope(grn_ctx *ctx, grn_rc rc, grn_obj *head, grn_obj *body, grn_obj *foot)
 {
@@ -305,7 +303,7 @@ do_alone(int argc, char **argv)
   char *path = NULL;
   grn_obj *db;
   grn_ctx ctx_, *ctx = &ctx_;
-  grn_ctx_init(ctx, (useql ? GRN_CTX_USE_QL : 0)|(batchmode ? GRN_CTX_BATCH_MODE : 0));
+  grn_ctx_init(ctx, 0);
   if (argc > 0 && argv) { path = *argv++; argc--; }
   db = (newdb || !path) ? grn_db_create(ctx, path, NULL) : grn_db_open(ctx, path);
   if (db) {
@@ -382,7 +380,7 @@ g_client(int argc, char **argv)
   grn_ctx ctx_, *ctx = &ctx_;
   const char *hostname = DEFAULT_DEST;
   if (argc > 0 && argv) { hostname = *argv++; argc--; }
-  grn_ctx_init(ctx, (batchmode ? GRN_CTX_BATCH_MODE : 0));
+  grn_ctx_init(ctx, 0);
   if (!grn_ctx_connect(ctx, hostname, port, 0)) {
     if (!argc) {
       grn_obj text;
@@ -733,41 +731,333 @@ h_output(grn_ctx *ctx, int flags, void *arg)
 }
 
 static void
-do_htreq(grn_ctx *ctx, grn_msg *msg)
+do_htreq_get(grn_ctx *ctx, grn_msg *msg)
 {
-  grn_sock fd = msg->u.fd;
-  grn_http_request_type t = grn_http_request_type_none;
-  grn_com_header *header = &msg->header;
-  switch (header->qtype) {
-  case 'G' : /* GET */
-    t = grn_http_request_type_get;
-    break;
-  case 'P' : /* POST */
-    t = grn_http_request_type_post;
-    break;
-  }
-  if (t) {
-    char *path = NULL;
-    char *pathe = GRN_BULK_HEAD((grn_obj *)msg);
-    char *e = GRN_BULK_CURR((grn_obj *)msg);
-    for (;; pathe++) {
-      if (e <= pathe + 6) {
-        /* invalid request */
-        goto exit;
-      }
-      if (*pathe == ' ') {
-        if (!path) {
-          path = pathe + 1;
-        } else {
-          if (!memcmp(pathe + 1, "HTTP/1", 6)) {
-            break;
-          }
+  char *path = NULL;
+  char *pathe = GRN_BULK_HEAD((grn_obj *)msg);
+  char *e = GRN_BULK_CURR((grn_obj *)msg);
+  for (;; pathe++) {
+    if (e <= pathe + 6) {
+      /* invalid request */
+      return;
+    }
+    if (*pathe == ' ') {
+      if (!path) {
+        path = pathe + 1;
+      } else {
+        if (!memcmp(pathe + 1, "HTTP/1", 6)) {
+          break;
         }
       }
     }
-    grn_ctx_send(ctx, path, pathe - path, 0);
   }
-exit :
+  grn_ctx_send(ctx, path, pathe - path, 0);
+}
+
+typedef struct {
+  const char *path_start;
+  int path_length;
+  int content_length;
+  grn_bool have_100_continue;
+  const char *body_start;
+} h_post_header;
+
+#define STRING_EQUAL(string, string_length, constant_string)\
+  (string_length == strlen(constant_string) &&\
+   strncmp(string, constant_string, string_length) == 0)
+
+#define STRING_EQUAL_CI(string, string_length, constant_string)\
+  (string_length == strlen(constant_string) &&\
+   strncasecmp(string, constant_string, string_length) == 0)
+
+static const char *
+do_htreq_post_parse_header_request_line(grn_ctx *ctx,
+                                        const char *start,
+                                        const char *end,
+                                        h_post_header *header)
+{
+  const char *current;
+
+  {
+    const char *method = start;
+    int method_length = -1;
+
+    for (current = method; current < end; current++) {
+      if (current[0] == '\n') {
+        return NULL;
+      }
+      if (current[0] == ' ') {
+        method_length = current - method;
+        current++;
+        break;
+      }
+    }
+    if (method_length == -1) {
+      return NULL;
+    }
+    if (!STRING_EQUAL_CI(method, method_length, "POST")) {
+      return NULL;
+    }
+  }
+
+  {
+    header->path_start = current;
+    header->path_length = -1;
+    for (; current < end; current++) {
+      if (current[0] == '\n') {
+        return NULL;
+      }
+      if (current[0] == ' ') {
+        header->path_length = current - header->path_start;
+        current++;
+        break;
+      }
+    }
+    if (header->path_length == -1) {
+      return NULL;
+    }
+  }
+
+  {
+    const char *http_version_start = current;
+    int http_version_length = -1;
+    for (; current < end; current++) {
+      if (current[0] == '\n') {
+        http_version_length = current - http_version_start;
+        if (http_version_length > 0 &&
+            http_version_start[http_version_length - 1] == '\r') {
+          http_version_length--;
+        }
+        current++;
+        break;
+      }
+    }
+    if (http_version_length == -1) {
+      return NULL;
+    }
+    if (!(STRING_EQUAL_CI(http_version_start, http_version_length, "HTTP/1.0") ||
+          STRING_EQUAL_CI(http_version_start, http_version_length, "HTTP/1.1"))) {
+      return NULL;
+    }
+  }
+
+  return current;
+}
+
+static const char *
+do_htreq_post_parse_header_values(grn_ctx *ctx,
+                                  const char *start,
+                                  const char *end,
+                                  h_post_header *header)
+{
+  const char *current;
+  const char *name = start;
+  int name_length = -1;
+  const char *value = NULL;
+  int value_length = -1;
+
+  for (current = start; current < end; current++) {
+    switch (current[0]) {
+    case '\n' :
+      if (name_length == -1) {
+        if (current - name == 1 && current[-1] == '\r') {
+          return current + 1;
+        } else {
+          /* No ":" header line. TODO: report error. */
+          return NULL;
+        }
+      } else {
+        while (value < current && value[0] == ' ') {
+          value++;
+        }
+        value_length = current - value;
+        if (value_length > 0 && value[value_length - 1] == '\r') {
+          value_length--;
+        }
+        if (STRING_EQUAL_CI(name, name_length, "Content-Length")) {
+          const char *rest;
+          header->content_length = grn_atoi(value, value + value_length, &rest);
+          if (rest != value + value_length) {
+            /* Invalid Content-Length value. TODO: report error. */
+            header->content_length = -1;
+          }
+        } else if (STRING_EQUAL_CI(name, name_length, "Expect")) {
+          if (STRING_EQUAL(value, value_length, "100-continue")) {
+            header->have_100_continue = GRN_TRUE;
+          }
+        }
+      }
+      name = current + 1;
+      name_length = -1;
+      value = NULL;
+      value_length = -1;
+      break;
+    case ':' :
+      if (name_length == -1) {
+        name_length = current - name;
+        value = current + 1;
+      }
+      break;
+    default :
+      break;
+    }
+  }
+
+  return NULL;
+}
+
+static grn_bool
+do_htreq_post_parse_header(grn_ctx *ctx,
+                           const char *start,
+                           const char *end,
+                           h_post_header *header)
+{
+  const char *current;
+
+  current = do_htreq_post_parse_header_request_line(ctx, start, end, header);
+  if (!current) {
+    return GRN_FALSE;
+  }
+  current = do_htreq_post_parse_header_values(ctx, current, end, header);
+  if (!current) {
+    return GRN_FALSE;
+  }
+
+  if (!header->have_100_continue && current == end) {
+    return GRN_FALSE;
+  }
+
+  if (current == end) {
+    header->body_start = NULL;
+  } else {
+    header->body_start = current;
+  }
+
+  return GRN_TRUE;
+}
+
+static void
+do_htreq_post(grn_ctx *ctx, grn_msg *msg)
+{
+  grn_sock fd = msg->u.fd;
+  const char *end;
+  h_post_header header;
+
+  header.path_start = NULL;
+  header.path_length = -1;
+  header.content_length = -1;
+  header.body_start = NULL;
+  header.have_100_continue = GRN_FALSE;
+
+  end = GRN_BULK_CURR((grn_obj *)msg);
+  if (!do_htreq_post_parse_header(ctx,
+                                  GRN_BULK_HEAD((grn_obj *)msg),
+                                  end,
+                                  &header)) {
+    return;
+  }
+
+  grn_ctx_send(ctx, header.path_start, header.path_length, GRN_CTX_QUIET);
+  if (ctx->rc != GRN_SUCCESS) {
+    ht_context context;
+    context.msg = msg;
+    h_output(ctx, GRN_CTX_TAIL, &context);
+    return;
+  }
+
+  if (header.have_100_continue) {
+    const char *continue_message = "HTTP/1.1 100 Continue\r\n";
+    ssize_t send_size;
+    int send_flags = MSG_NOSIGNAL;
+    send_size = send(fd, continue_message, strlen(continue_message), send_flags);
+    if (send_size == -1) {
+      SERR("send");
+      return;
+    }
+  }
+
+  {
+    grn_obj line_buffer;
+    int read_content_length = 0;
+
+    GRN_TEXT_INIT(&line_buffer, 0);
+    while (read_content_length < header.content_length) {
+#define POST_BUFFER_SIZE 8192
+      grn_rc rc;
+      char buffer[POST_BUFFER_SIZE];
+      const char *buffer_start, *buffer_current, *buffer_end;
+
+      if (header.body_start) {
+        buffer_start = header.body_start;
+        buffer_end = end;
+        header.body_start = NULL;
+      } else {
+        ssize_t recv_length;
+        int recv_flags = 0;
+        recv_length = recv(fd, buffer, POST_BUFFER_SIZE, recv_flags);
+        if (recv_length == 0) {
+          break;
+        }
+        if (recv_length == -1) {
+          SERR("recv");
+          break;
+        }
+        buffer_start = buffer;
+        buffer_end = buffer_start + recv_length;
+      }
+      read_content_length += buffer_end - buffer_start;
+
+      rc = GRN_SUCCESS;
+      buffer_current = buffer_start;
+      for (; rc == GRN_SUCCESS && buffer_current < buffer_end; buffer_current++) {
+        if (buffer_current[0] != '\n') {
+          continue;
+        }
+        GRN_TEXT_PUT(ctx,
+                     &line_buffer,
+                     buffer_start,
+                     buffer_current - buffer_start);
+        {
+          int flags = 0;
+          if (!(read_content_length == header.content_length &&
+                buffer_current + 1 == buffer_end)) {
+            flags |= GRN_CTX_QUIET;
+          }
+          rc = grn_ctx_send(ctx,
+                            GRN_TEXT_VALUE(&line_buffer),
+                            GRN_TEXT_LEN(&line_buffer),
+                            flags);
+        }
+        buffer_start = buffer_current + 1;
+        GRN_BULK_REWIND(&line_buffer);
+      }
+      GRN_TEXT_PUT(ctx, &line_buffer, buffer_start, buffer_end - buffer_start);
+#undef POST_BUFFER_SIZE
+    }
+
+    if (GRN_TEXT_LEN(&line_buffer) > 0) {
+      grn_ctx_send(ctx,
+                   GRN_TEXT_VALUE(&line_buffer),
+                   GRN_TEXT_LEN(&line_buffer),
+                   0);
+    }
+
+    GRN_OBJ_FIN(ctx, &line_buffer);
+  }
+}
+
+static void
+do_htreq(grn_ctx *ctx, grn_msg *msg)
+{
+  grn_com_header *header = &msg->header;
+  switch (header->qtype) {
+  case 'G' : /* GET */
+    do_htreq_get(ctx, msg);
+    break;
+  case 'P' : /* POST */
+    do_htreq_post(ctx, msg);
+    break;
+  }
   /* TODO: support "Connection: keep-alive" */
   ctx->stat = GRN_CTX_QUIT;
   /* TODO: support a command in multi requests. e.g.: load command */
@@ -775,7 +1065,7 @@ exit :
   /* if (ctx->rc != GRN_OPERATION_WOULD_BLOCK) {...} */
   grn_msg_close(ctx, (grn_obj *)msg);
   /* if not keep alive connection */
-  grn_sock_close(fd);
+  grn_sock_close(msg->u.fd);
   grn_com_event_start_accept(ctx, msg->acceptor->ev);
 }
 
@@ -1562,7 +1852,7 @@ g_handler(grn_ctx *ctx, grn_obj *msg)
     int added;
     edge = grn_edges_add(ctx, &((grn_msg *)msg)->edge_id, &added);
     if (added) {
-      grn_ctx_init(&edge->ctx, (useql ? GRN_CTX_USE_QL : 0));
+      grn_ctx_init(&edge->ctx, 0);
       GRN_COM_QUEUE_INIT(&edge->recv_new);
       GRN_COM_QUEUE_INIT(&edge->send_old);
       grn_ctx_use(&edge->ctx, (grn_obj *)com->ev->opaque);
@@ -1612,7 +1902,6 @@ enum {
 };
 
 #define MODE_MASK   0x007f
-#define MODE_USE_QL 0x0080
 #define MODE_NEW_DB 0x0100
 
 static uint32_t
@@ -1830,7 +2119,8 @@ config_file_load(const char *path, const grn_str_getopt_opt *opts, int *flags)
   return status;
 }
 
-static const int default_port = DEFAULT_PORT;
+static const int default_http_port = DEFAULT_HTTP_PORT;
+static const int default_gqtp_port = DEFAULT_GQTP_PORT;
 static grn_encoding default_encoding = GRN_ENC_DEFAULT;
 static uint32_t default_max_num_threads = DEFAULT_MAX_NFTHREADS;
 static const int default_mode = mode_alone;
@@ -2039,7 +2329,8 @@ show_usage(FILE *output)
           "      --bind-address <ip/hostname>:\n"
           "                                specify server address to bind\n"
           "                                (default: %s)\n"
-          "  -p, --port <port number>:     specify server port number (default: %d)\n"
+          "  -p, --port <port number>:     specify server port number\n"
+          "                                (HTTP default: %d, GQTP default: %d)\n"
           "  -i, --server-id <ip/hostname>:\n"
           "                                specify server ID address (default: %s)\n"
           "      --protocol <protocol>:    specify server protocol to listen\n"
@@ -2083,8 +2374,8 @@ show_usage(FILE *output)
           "  <db pathname>: in server/daemon mode\n"
           "  <dest hostname> [<commands>]: in client mode (default: %s)\n",
           grn_encoding_to_string(default_encoding),
-          default_port, default_bind_address,
-          default_port, default_hostname, default_protocol,
+          default_gqtp_port, default_bind_address,
+          default_http_port, default_gqtp_port, default_hostname, default_protocol,
           default_document_root, default_cache_limit, default_max_num_threads,
           default_log_level, default_log_path, default_query_log_path,
           default_config_path, default_default_command_version,
@@ -2118,7 +2409,6 @@ main(int argc, char **argv)
     {'s', NULL, NULL, mode_server, GETOPT_OP_UPDATE},
     {'l', "log-level", NULL, 0, GETOPT_OP_NONE},
     {'i', "server-id", NULL, 0, GETOPT_OP_NONE},
-    {'q', NULL, NULL, MODE_USE_QL, GETOPT_OP_ON},
     {'n', NULL, NULL, MODE_NEW_DB, GETOPT_OP_ON},
     {'\0', "protocol", NULL, 0, GETOPT_OP_NONE},
     {'\0', "version", NULL, mode_version, GETOPT_OP_UPDATE},
@@ -2143,20 +2433,20 @@ main(int argc, char **argv)
   opts[2].arg = &max_num_threads_arg;
   opts[7].arg = &log_level_arg;
   opts[8].arg = &hostname_arg;
-  opts[11].arg = &protocol_arg;
-  opts[13].arg = &log_path_arg;
-  opts[14].arg = &query_log_path_arg;
-  opts[15].arg = &pid_file_path;
-  opts[16].arg = &config_path;
-  opts[18].arg = &cache_limit_arg;
-  opts[19].arg = &input_path;
-  opts[20].arg = &document_root_arg;
-  opts[21].arg = &default_command_version_arg;
-  opts[22].arg = &default_match_escalation_threshold_arg;
-  opts[23].arg = &bind_address_arg;
-  opts[24].arg = &input_fd_arg;
-  opts[25].arg = &output_fd_arg;
-  opts[26].arg = &working_directory_arg;
+  opts[10].arg = &protocol_arg;
+  opts[12].arg = &log_path_arg;
+  opts[13].arg = &query_log_path_arg;
+  opts[14].arg = &pid_file_path;
+  opts[15].arg = &config_path;
+  opts[17].arg = &cache_limit_arg;
+  opts[18].arg = &input_path;
+  opts[19].arg = &document_root_arg;
+  opts[20].arg = &default_command_version_arg;
+  opts[21].arg = &default_match_escalation_threshold_arg;
+  opts[22].arg = &bind_address_arg;
+  opts[23].arg = &input_fd_arg;
+  opts[24].arg = &output_fd_arg;
+  opts[25].arg = &working_directory_arg;
 
   reset_ready_notify_pipe();
 
@@ -2231,7 +2521,11 @@ main(int argc, char **argv)
     }
     port = value;
   } else {
-    port = default_port;
+    if (protocol_arg) {
+      if (*protocol_arg == 'h' || *protocol_arg == 'H') {
+        port = default_http_port;
+      }
+    }
   }
 
   if (encoding_arg) {
@@ -2494,7 +2788,6 @@ main(int argc, char **argv)
   }
 
   newdb = (mode & MODE_NEW_DB);
-  useql = (mode & MODE_USE_QL);
   switch (mode & MODE_MASK) {
   case mode_alone :
     exit_code = do_alone(argc - i, argv + i);
